@@ -1395,7 +1395,7 @@ CRQ_CallsignPage::validate() {
 #ifdef DEBUG
 	DXCC dx;
 #endif
-	if (!showAll) {
+	if (!showAll && _parent->_crq == NULL) {
 		prefixMap::iterator it;
 		for (it = _parent->prefixRegex.begin(); it != _parent->prefixRegex.end(); it++) {
 #ifndef DEBUG
